import { Action, ActionCreator, AnyAction } from "typescript-fsa";
export interface ReducerBuilder<InS, OutS = InS, PassedS = InS | undefined> {
    case<P>(actionCreator: ActionCreator<P>, handler: Handler<InS, OutS, P>): ReducerBuilder<InS, OutS, PassedS>;
    caseWithAction<P>(actionCreator: ActionCreator<P>, handler: Handler<InS, OutS, Action<P>>): ReducerBuilder<InS, OutS, PassedS>;
    cases<P1, P2>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>], handler: Handler<InS, OutS, P1 | P2>): ReducerBuilder<InS, OutS, PassedS>;
    cases<P1, P2, P3>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>], handler: Handler<InS, OutS, P1 | P2 | P3>): ReducerBuilder<InS, OutS, PassedS>;
    cases<P1, P2, P3, P4>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>, ActionCreator<P4>], handler: Handler<InS, OutS, P1 | P2 | P3 | P4>): ReducerBuilder<InS, OutS, PassedS>;
    cases<P>(actionCreators: Array<ActionCreator<P>>, handler: Handler<InS, OutS, P>): ReducerBuilder<InS, OutS, PassedS>;
    casesWithAction<P1, P2>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>], handler: Handler<InS, OutS, Action<P1 | P2>>): ReducerBuilder<InS, OutS, PassedS>;
    casesWithAction<P1, P2, P3>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>], handler: Handler<InS, OutS, Action<P1 | P2 | P3>>): ReducerBuilder<InS, OutS, PassedS>;
    casesWithAction<P1, P2, P3, P4>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>, ActionCreator<P4>], handler: Handler<InS, OutS, Action<P1 | P2 | P3 | P4>>): ReducerBuilder<InS, OutS, PassedS>;
    casesWithAction<P>(actionCreators: Array<ActionCreator<P>>, handler: Handler<InS, OutS, Action<P>>): ReducerBuilder<InS, OutS, PassedS>;
    withHandling(updateBuilder: (builder: ReducerBuilder<InS, OutS, PassedS>) => ReducerBuilder<InS, OutS, PassedS>): ReducerBuilder<InS, OutS, PassedS>;
    default(defaultHandler: Handler<InS, OutS, AnyAction>): (state: PassedS, action: {
        type: any;
    }) => OutS;
    build(): (state: PassedS, action: {
        type: any;
    }) => OutS;
    (state: PassedS, action: AnyAction): OutS;
}
export declare type Handler<InS, OutS, P> = (state: InS, payload: P) => OutS;
export declare function reducerWithInitialState<S>(initialState: S): ReducerBuilder<S>;
export declare function reducerWithoutInitialState<S>(): ReducerBuilder<S, S, S>;
export declare function upcastingReducer<InS extends OutS, OutS>(): ReducerBuilder<InS, OutS, InS>;
